﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "LMGrabRange.h"

#include "LMItemDrop.h"
#include "Player/LMPlayer.h"


ULMGrabRange::ULMGrabRange()
{
	PrimaryComponentTick.bCanEverTick = true;

}

void ULMGrabRange::TickComponent(float DeltaTime, enum ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	if (!bIsEnabled)
		return;

	for (int i = Drops.Num() - 1; i >= 0; i--)
	{
		ALMItemDrop* Item = Drops[i];
		
		const FVector ItemPosition = Item->GetActorLocation();
		const FVector Position = GetComponentLocation();

		const float DistanceLeft = FVector::Dist(ItemPosition, Position);
		// FString text = FString::Printf(TEXT("Item %s at distance %f"), *Item->GetName(), DistanceLeft);
		// GEngine->AddOnScreenDebugMessage(i, 1.f, FColor::Cyan, text);
		
		if (DistanceLeft <= RangeToGrab)
		{
			Item->ConsumePack(Player);
			Drops.Remove(Item);
		}
	}
}

void ULMGrabRange::OnDropOverlapped(ALMItemDrop* ItemDrop)
{
	Drops.AddUnique(ItemDrop);
	ItemDrop->SetAttractor(Player);
}

void ULMGrabRange::BeginPlay()
{
	Super::BeginPlay();
	Player = CastChecked<ALMPlayer>(GetOwner());
	OnComponentBeginOverlap.AddUniqueDynamic(this, &ULMGrabRange::OnComponentOverlap);
}

void ULMGrabRange::OnComponentOverlap(UPrimitiveComponent* OverlappedComponent, AActor* OtherActor,
	UPrimitiveComponent* OtherComp, int32 OtherBodyIndex, bool bFromSweep, const FHitResult& SweepResult)
{
	if (!bIsEnabled)
		return;
	
	if (ALMItemDrop* ItemDrop = Cast<ALMItemDrop>(OtherActor))
	{
		OnDropOverlapped(ItemDrop);
	}
}

